<?php

/**
 * Table Of Contents
 *
 * Speaker_Theme_Aq_Resize ( Image resizing class )
 * speaker_theme_aq_resize ( Image resizing function )
 * speaker_theme_get_theme_mod ( Returns customizer option value )
 * speaker_theme_get_post_meta ( Returns post meta value )
 * speaker_theme_get_term_meta ( Return term meta value )
 * speaker_theme_get_attachment_alt ( Returns the alt attribute for an attachment )
 * speaker_theme_set_post_vars ( Set global post vars )
 * speaker_theme_get_post_vars ( Get global post vars )
 * speaker_theme_get_sidebar_width_class ( Get sidebar width class )
 * speaker_theme_get_content_width_class ( Get content width class )
 * speaker_theme_is_logo_shown ( is a logo shown )
 * speaker_theme_get_image_size ( get image size in specific column )
 * speaker_theme_get_default_bg_color ( default body bg color )
 * speaker_theme_has_custom_header ( does the post/page have a custom header image )
 */

if( ! class_exists('Speaker_Theme_Aq_Resize') ) {

	/**
	 * Image resizing class
	 *
	 * @since 1.0
	 */
	class Speaker_Theme_Aq_Resize {

		/**
		 * The singleton instance
		 */
		static private $instance = null;

		/**
		 * No initialization allowed
		 */
		private function __construct() {}

		/**
		 * No cloning allowed
		 */
		private function __clone() {}

		/**
		 * For your custom default usage you may want to initialize an Aq_Resize object by yourself and then have own defaults
		 */
		static public function getInstance() {
			if(self::$instance == null) {
				self::$instance = new self;
			}

			return self::$instance;
		}

		/**
		 * Run, forest.
		 */
		public function process( $url, $width = null, $height = null, $crop = null, $single = true, $upscale = true ) {

			// Validate inputs.
			if ( ! $url || ( ! $width && ! $height ) ) return false;

			$upscale = true;

			// Caipt'n, ready to hook.
			if ( true === $upscale ) add_filter( 'image_resize_dimensions', array($this, 'aq_upscale'), 10, 6 );

			// Define upload path & dir.
			$upload_info = wp_upload_dir();
			$upload_dir = $upload_info['basedir'];
			$upload_url = $upload_info['baseurl'];
			
			$http_prefix = "http://";
			$https_prefix = "https://";
			
			/* if the $url scheme differs from $upload_url scheme, make them match 
			   if the schemes differe, images don't show up. */
			if(!strncmp($url,$https_prefix,strlen($https_prefix))){ //if url begins with https:// make $upload_url begin with https:// as well
				$upload_url = str_replace($http_prefix,$https_prefix,$upload_url);
			}
			elseif(!strncmp($url,$http_prefix,strlen($http_prefix))){ //if url begins with http:// make $upload_url begin with http:// as well
				$upload_url = str_replace($https_prefix,$http_prefix,$upload_url);      
			}
			

			// Check if $img_url is local.
			if ( false === strpos( $url, $upload_url ) ) return false;

			// Define path of image.
			$rel_path = str_replace( $upload_url, '', $url );
			$img_path = $upload_dir . $rel_path;

			// Check if img path exists, and is an image indeed.
			if ( ! file_exists( $img_path ) or ! getimagesize( $img_path ) ) return false;

			// Get image info.
			$info = pathinfo( $img_path );
			$ext = $info['extension'];
			list( $orig_w, $orig_h ) = getimagesize( $img_path );

			// Get image size after cropping.
			$dims = image_resize_dimensions( $orig_w, $orig_h, $width, $height, $crop );
			$dst_w = $dims[4];
			$dst_h = $dims[5];

			// Return the original image only if it exactly fits the needed measures.
			if ( ! $dims && ( ( ( null === $height && $orig_w == $width ) xor ( null === $width && $orig_h == $height ) ) xor ( $height == $orig_h && $width == $orig_w ) ) ) {
				$img_url = $url;
				$dst_w = $orig_w;
				$dst_h = $orig_h;
			} else {
				// Use this to check if cropped image already exists, so we can return that instead.
				$suffix = "{$dst_w}x{$dst_h}";
				$dst_rel_path = str_replace( '.' . $ext, '', $rel_path );
				$destfilename = "{$upload_dir}{$dst_rel_path}-{$suffix}.{$ext}";

				if ( ! $dims || ( true == $crop && false == $upscale && ( $dst_w < $width || $dst_h < $height ) ) ) {
					// Can't resize, so return false saying that the action to do could not be processed as planned.
					return $url;
				}
				// Else check if cache exists.
				elseif ( file_exists( $destfilename ) && getimagesize( $destfilename ) ) {
					$img_url = "{$upload_url}{$dst_rel_path}-{$suffix}.{$ext}";
				}
				// Else, we resize the image and return the new resized image url.
				else {

					$editor = wp_get_image_editor( $img_path );

					if ( is_wp_error( $editor ) || is_wp_error( $editor->resize( $width, $height, $crop ) ) )
						return $url;

					$resized_file = $editor->save();

					if ( ! is_wp_error( $resized_file ) ) {
						$resized_rel_path = str_replace( $upload_dir, '', $resized_file['path'] );
						$img_url = $upload_url . $resized_rel_path;
					} else {
						return $url;
					}

				}
			}

			// Okay, leave the ship.
			if ( true === $upscale ) remove_filter( 'image_resize_dimensions', array( $this, 'aq_upscale' ) );

			// Return the output.
			if ( $single ) {
				// str return.
				$image = $img_url;
			} else {
				// array return.
				$image = array (
					0 => $img_url,
					1 => $dst_w,
					2 => $dst_h
				);
			}

			return $image;
		}

		/**
		 * Callback to overwrite WP computing of thumbnail measures
		 */
		function aq_upscale( $default, $orig_w, $orig_h, $dest_w, $dest_h, $crop ) {
			if ( ! $crop ) return null; // Let the wordpress default function handle this.

			// Here is the point we allow to use larger image size than the original one.
			$aspect_ratio = $orig_w / $orig_h;
			$new_w = $dest_w;
			$new_h = $dest_h;

			if ( ! $new_w ) {
				$new_w = intval( $new_h * $aspect_ratio );
			}

			if ( ! $new_h ) {
				$new_h = intval( $new_w / $aspect_ratio );
			}

			$size_ratio = max( $new_w / $orig_w, $new_h / $orig_h );

			$crop_w = round( $new_w / $size_ratio );
			$crop_h = round( $new_h / $size_ratio );

			$s_x = floor( ( $orig_w - $crop_w ) / 2 );
			$s_y = floor( ( $orig_h - $crop_h ) / 2 );

			return array( 0, 0, (int) $s_x, (int) $s_y, (int) $new_w, (int) $new_h, (int) $crop_w, (int) $crop_h );
		}

	}
	
}


if ( ! function_exists('speaker_theme_aq_resize') ) {

	/**
	 * Resize an image using Speaker_Theme_Aq_Resize Class
	 *
	 * @since 1.0
	 *
	 * @param string $url     The URL of the image
	 * @param int    $width   The new width of the image
	 * @param int    $height  The new height of the image
	 * @param bool   $crop    To crop or not to crop, the question is now
	 * @param bool   $single  If true only returns the URL, if false returns array
	 * @param bool   $upscale If image not big enough for new size should it upscale
	 * @return mixed If $single is true return new image URL, if it is false return array
	 *               Array contains 0 = URL, 1 = width, 2 = height
	 */
	function speaker_theme_aq_resize( $url, $width = null, $height = null, $crop = null, $single = true, $upscale = false ) {

		 if( class_exists( 'Jetpack' ) && Jetpack::is_module_active( 'photon' ) ) {

			$args = array(
				'resize' => "$width,$height"
			);
			if ( $single == true ) {
				return jetpack_photon_url( $url, $args );
			} else {
				$image = array (
					0 => $img_url,
					1 => $width,
					2 => $height
				);
				return jetpack_photon_url( $url, $args );
			}

		} else {

			$aq_resize = Speaker_Theme_Aq_Resize::getInstance();
			return $aq_resize->process( $url, $width, $height, $crop, $single, $upscale );
			
		}

	}

}

/**
 * returns customizer option value
 *
 * @since 1.0
 */
function speaker_theme_get_theme_mod( $option_id, $default = '' ) {

	$return = get_theme_mod( 'speaker_theme_' . $option_id, $default );
	if ( $return === '' ) { $return = $default; }

	if ( $option_id == 'general_layout' && is_page_template( 'template-sections.php' ) ) {
		$return = 'full';
	}

	return $return;

}

/**
 * returns post meta value
 *
 * @since 1.0
 */
function speaker_theme_get_post_meta( $post_id, $option_id ) {

	$option_id = '_speaker_theme_' . $option_id;

	return get_post_meta( $post_id, $option_id , true );

}

/**
 * returns term meta value
 *
 * @since 1.0
 */
function speaker_theme_get_term_meta( $term_id, $option_id ) {

	$option_id = '_speaker_theme_' . $option_id;

	return get_term_meta( $term_id, $option_id, true );

}

/**
 * returns the ALT attribute for an attachment
 *
 * @since 1.0
 *
 * @param string   $attachment_ID The ID of the attachment
 * @return string  The ALT attribute text
 */
function speaker_theme_get_attachment_alt( $attachment_ID ) {

	// Get ALT
	$thumb_alt = trim( strip_tags( get_post_meta( $attachment_ID, '_wp_attachment_image_alt', true) ) );
	
	// No ALT supplied get attachment info
	if ( empty( $thumb_alt ) )
		$attachment = get_post( $attachment_ID );
	
	// Use caption if no ALT supplied
	if ( empty( $thumb_alt ) )
		$thumb_alt = trim(strip_tags( $attachment->post_excerpt ));
	
	// Use title if no caption supplied either
	if ( empty( $thumb_alt ) )
		$thumb_alt = trim(strip_tags( $attachment->post_title ));

	// Return ALT
	return esc_attr( $thumb_alt );

}

/**
 * set global post vars
 *
 * @since 1.0
 */
function speaker_theme_set_post_vars( $post_vars ) {
	global $speaker_theme_post_vars;
	$speaker_theme_post_vars = $post_vars;
}

/**
 * get global post vars
 *
 * @since 1.0
 */
function speaker_theme_get_post_vars() {
	global $speaker_theme_post_vars;
	return $speaker_theme_post_vars;
}

/**
 * Get sidebar width class
 *
 * @since 1.0
 */
function speaker_theme_get_sidebar_width_class() {

	// vars
	$sidebar_class = '';

	// get the sidebar width
	$sidebar_width = speaker_theme_get_theme_mod( 'sidebar_width', '4' );

	// pass back the class
	$sidebar_class = 'col col-' . $sidebar_width . ' col-last';

	// sidebar border
	if ( speaker_theme_get_theme_mod( 'sidebar_border', 1 ) == 1 ){
		$sidebar_class .= ' with-border';
	}

	return $sidebar_class;

}

/**
 * Get content width class
 *
 * @since 1.0
 */
function speaker_theme_get_content_width_class() {

	// get the content width
	$sidebar_width = speaker_theme_get_theme_mod( 'sidebar_width', '4' );
	$content_width = 12 - $sidebar_width;

	// blog layout
	$blog_layout = speaker_theme_get_theme_mod( 'blog_layout', 'content_sidebar' );

	// if blog layout without sidebar, no column class
	if ( $blog_layout == 'full_content' && ( is_single() || is_archive() || is_search() || is_home() ) ) {
		return '';
	}

	// pass back the class
	return 'col col-' . $content_width;

}

/**
 * Get container width
 *
 * @since 1.0
 */
function speaker_theme_get_container_width() {

	return speaker_theme_get_theme_mod( 'container_width', '1200' );

}

/**
 * Is a logo shown
 *
 * @since 1.0
 */
function speaker_theme_is_logo_shown() {

	$custom_logo = get_theme_mod( 'custom_logo', false );
	$text_logo_title = speaker_theme_get_theme_mod( 'logo_display_title', 1 );
	$text_logo_tagline = speaker_theme_get_theme_mod( 'logo_display_tagline', 0 );

	if ( $custom_logo == false && $text_logo_title == 0 && $text_logo_tagline == 0 ) {
		return false;
	} else {
		return true;
	}

}

/**
 * Get image size in specfic column
 *
 * @since 1.0
 */
function speaker_theme_get_image_size( $column = 6, $with_sidebar = false ) {

	// container size
	$full_size = speaker_theme_get_container_width();

	// columns percentages
	$col_spacing = 3.42;
	$col_1 = 5.198;
	$col_2 = 13.81;
	$col_3 = 22.43;
	$col_4 = 31.05;
	$col_5 = 39.67;
	$col_6 = 48.29;
	$col_7 = 56.90;
	$col_8 = 65.52;
	$col_9 = 74.14;
	$col_10 = 82.76;
	$col_11 = 91.38;
	$col_12 = 100;

	// if sidebar container size is smaller
	if ( $with_sidebar ) {
		$sidebar_col_var = 'col_' . speaker_theme_get_theme_mod( 'sidebar_width', '4' );
		$full_size = $full_size / 100 * ( 100 - $col_spacing - $$sidebar_col_var );
	}

	// get column var
	$col_var = 'col_' . $column;
	$col = $$col_var;

	// get image size
	$image_size =  round( $full_size / 100 * $col );	

	// pass it back
	return $image_size;

}

/**
 * Get default body bg color
 *
 * @since 1.0
 */
function speaker_theme_get_default_bg_color() {

	if ( speaker_theme_get_theme_mod( 'general_layout', 'full' ) == 'wrapped' ) {
		return '#f1f3f6';
	} else {
		return '#fff';
	}

}

/**
 * Does the post/page have a custom header image
 *
 * @since 1.0.2
 */
function speaker_theme_has_custom_header() {

	// default to false
	$has_custom_header = false;

	// if not post or page return false
	if ( ! is_singular( array( 'post', 'page' ) ) ) {
		return false;
	}

	if ( speaker_theme_get_post_meta( get_the_ID(), 'custom_header_image' ) ) {
		$has_custom_header = true;
	}

	return $has_custom_header;

}